lexer grammar PrevLexer;

@header {
	package prev.phase.lexan;
	import prev.common.report.*;
	import prev.data.sym.*;
}

@members {
    @Override
	public Token nextToken() {
		return (Token) super.nextToken();
	}
}

CONST_VOID : 'none';
CONST_BOOL : ('true'|'false');
CONST_INT  : ('0' | [1-9]+ [0-9]*);
CONST_CHAR : '\''([ -&]|[(-~]|'\\\'')'\'';
CONST_STR  : '"'([ -!]|[#-~]|'\\"')*'"';
CONST_PTR  : 'nil';

OPEN_BRACK   : '(';
OPEN_CBRACK  : '{';
OPEN_SBRACK  : '[';
CLOSE_BRACK  : ')';
CLOSE_CBRACK : '}';
CLOSE_SBRACK : ']';

DOT 	   	: '.';
COMMA 	   	: ',';
COLON      	: ':'; 
SEMI_COLON 	: ';';
ANDSYM	 	: '&';
PIPE	   	: '|';
EXCLAMATION	: '!';
IS_EQUAL 	: '==';
NOT_EQUAL	: '!=';
LESS_THAN	: '<';
GREATER_THAN : '>';
LESS_OR_EQUAL : '<=';
GREATER_OR_EQUAL : '>=';

STAR		: '*';
SLASH		: '/';
PERCENT		: '%';
PLUS		: '+';
MINUS		: '-';
CARROT		: '^';
EQUALS		: '=';

BOOL	: 'bool';
CHAR	: 'char';
DEL		: 'del';
DO		: 'do';
ELSE	: 'else';
FUN		: 'fun';
IF		: 'if';
INT		: 'int';
NEW		: 'new';
THEN	: 'then';
TYP		: 'typ';
VAR		: 'var';
VOID	: 'void';
WHERE	: 'where';
WHILE	: 'while';

IDENTIFIER : [a-zA-Z_]+[a-zA-Z0-9_]*;
COMMENT	: '#' ~('\r' | '\n')* -> skip;
WHITESPACE : [ \n\r]+ -> skip;
TAB : '\t' { if (true) { 
	int curr = getCharPositionInLine();
	int off = ((curr / 8) + 1) * 8;
	this.setCharPositionInLine(off); 
}} -> skip;

CONST_INT_ERROR  : ('0')+[0-9]* { if (true) {new Report.Error(new Location(_tokenStartLine, _tokenStartCharPositionInLine), "Invalid integer: " + getText()); }};
CONST_CHAR_ERROR : ('\'' '\'') | ('\''([ -&]|[(-~])+'\'') | ('\'') { if (true) {new Report.Error(new Location(_tokenStartLine, _tokenStartCharPositionInLine), "Invalid character: " + getText()); }};
CONST_STR_ERROR  : ('"') { if (true) {new Report.Error(new Location(_tokenStartLine, _tokenStartCharPositionInLine), "Invalid string: " + getText()); }};

ERROR: . { if (true) {new Report.Error(new Location(_tokenStartLine, _tokenStartCharPositionInLine), "Error: " + getText()); }};