parser grammar PrevParser;

@header {

	package prev.phase.synan;
	
	import java.util.*;
	
	import prev.common.report.*;
	import prev.data.ast.tree.*;
	import prev.data.ast.tree.decl.*;
	import prev.data.ast.tree.expr.*;
	import prev.data.ast.tree.stmt.*;
	import prev.data.ast.tree.type.*;
	import prev.phase.lexan.*;
	
}

@members {

	private Location loc(Token     tok) { return new Location((prev.data.sym.Token)tok); }
	private Location loc(Locatable loc) { return new Location(loc                 ); }
	private Location loc(Token     tok1, Token     tok2) { return new Location((prev.data.sym.Token)tok1, (prev.data.sym.Token)tok2); }
	private Location loc(Token     tok1, Locatable loc2) { return new Location((prev.data.sym.Token)tok1, loc2); }
	private Location loc(Locatable loc1, Token     tok2) { return new Location(loc1,                      (prev.data.sym.Token)tok2); }
	private Location loc(Locatable loc1, Locatable loc2) { return new Location(loc1,                      loc2); }

}

options{
    tokenVocab=PrevLexer;
}


source returns [AstTrees<AstDecl> ast]
	: { Vector<AstDecl> decls = new Vector<>(); } 
		(decl { decls.add($decl.ast); })+ 
		{ $ast = new AstTrees<AstDecl>(decls); } EOF
	;

decl returns [AstDecl ast]
	: TYP IDENTIFIER EQUALS type
	{ $ast = new AstTypeDecl(loc($TYP, $type.ast), $IDENTIFIER.getText(), $type.ast); }

	| VAR IDENTIFIER COLON type
	{ $ast = new AstVarDecl(loc($VAR, $type.ast), $IDENTIFIER.getText(), $type.ast); }

	// Function declaration
	| FUN IDENTIFIER OPEN_BRACK
	{ Vector<AstParDecl> parameters = new Vector<>(); }
	{ String name = $IDENTIFIER.getText(); }
	(IDENTIFIER COLON type 
	{ parameters.add(new AstParDecl(loc($IDENTIFIER, $type.ast), $IDENTIFIER.getText(), $type.ast)); }
	(COMMA IDENTIFIER COLON type
	{ parameters.add(new AstParDecl(loc($IDENTIFIER, $type.ast), $IDENTIFIER.getText(), $type.ast)); }
	)*)? CLOSE_BRACK COLON 
	{ AstTrees<AstParDecl> treeFunPar = parameters.size() > 0 ? new AstTrees<AstParDecl>(parameters) : null; }
	type 
	{ Location location = loc($FUN, $type.ast); }
	{ AstExpr astExpr = null; }
	(EQUALS expr
	{ location = loc($FUN, $expr.ast); } 
	{ astExpr = $expr.ast; }
	)?
	{ $ast = new AstFunDecl(location, name, treeFunPar, $type.ast, astExpr); }
	;

type returns [AstType ast]
	: VOID { $ast = new AstAtomType(loc($VOID), AstAtomType.Type.VOID); }
	| CHAR { $ast = new AstAtomType(loc($CHAR), AstAtomType.Type.CHAR); }
	| INT  { $ast = new AstAtomType(loc($INT), AstAtomType.Type.INT); }
	| BOOL { $ast = new AstAtomType(loc($BOOL), AstAtomType.Type.BOOL); }
	| IDENTIFIER { $ast = new AstNameType(loc($IDENTIFIER), $IDENTIFIER.getText()); }
	| OPEN_SBRACK expr CLOSE_SBRACK type { $ast = new AstArrType(loc($OPEN_SBRACK, $type.ast), $type.ast, $expr.ast); }
	| CARROT type { $ast = new AstPtrType(loc($CARROT, $type.ast), $type.ast); }

	| OPEN_CBRACK IDENTIFIER COLON type 
	{ Vector<AstCompDecl> comps = new Vector<>(); }
	{ comps.add(new AstCompDecl(loc($IDENTIFIER, $type.ast), $IDENTIFIER.getText(), $type.ast)); }
	(COMMA IDENTIFIER COLON type
	{ comps.add(new AstCompDecl(loc($IDENTIFIER, $type.ast), $IDENTIFIER.getText(), $type.ast)); }
	)* CLOSE_CBRACK
	{ AstTrees<AstCompDecl> tree = comps.size() > 0 ? new AstTrees<AstCompDecl>(comps) : null; }
	{ $ast = new AstRecType(loc($OPEN_CBRACK, $CLOSE_CBRACK), tree); }

	| OPEN_BRACK type CLOSE_BRACK
	{ $ast = $type.ast; }
	{ $ast.relocate(loc($OPEN_BRACK, $CLOSE_BRACK)); }
	;

expr returns [AstExpr ast]
	: CONST_VOID { $ast = new AstAtomExpr(loc($CONST_VOID), AstAtomExpr.Type.VOID, $CONST_VOID.getText()); } 
	| CONST_BOOL { $ast = new AstAtomExpr(loc($CONST_BOOL), AstAtomExpr.Type.BOOL, $CONST_BOOL.getText()); }
	| CONST_INT  { $ast = new AstAtomExpr(loc($CONST_INT), AstAtomExpr.Type.INT, $CONST_INT.getText()); }
	| CONST_CHAR { $ast = new AstAtomExpr(loc($CONST_CHAR), AstAtomExpr.Type.CHAR, $CONST_CHAR.getText()); }
	| CONST_STR  { $ast = new AstAtomExpr(loc($CONST_STR), AstAtomExpr.Type.STRING, $CONST_STR.getText()); }
	| CONST_PTR  { $ast = new AstAtomExpr(loc($CONST_PTR), AstAtomExpr.Type.POINTER, $CONST_PTR.getText()); }
	| IDENTIFIER { $ast = new AstNameExpr(loc($IDENTIFIER), $IDENTIFIER.getText()); }
	
	// Function call
	| IDENTIFIER OPEN_BRACK 
	{ Vector<AstExpr> exprs = new Vector<>(); }
	(expr { exprs.add($expr.ast); } 
	(COMMA expr { exprs.add($expr.ast); } 
	)*)? CLOSE_BRACK
	{ AstTrees<AstExpr> treeExprs = exprs.size() > 0 ? new AstTrees<AstExpr>(exprs) : null; }
	{ $ast = new AstCallExpr(loc($IDENTIFIER, $CLOSE_BRACK), $IDENTIFIER.getText(), treeExprs); }

	// Compound 
	| OPEN_CBRACK 
	{ Vector<AstStmt> statements = new Vector<>(); }
	(stmt SEMI_COLON { statements.add($stmt.ast); })* CLOSE_CBRACK
	{ $ast = new AstStmtExpr(loc($OPEN_CBRACK, $CLOSE_CBRACK), new AstTrees<AstStmt>(statements)); }

	// Typecast
	| OPEN_BRACK expr COLON type CLOSE_BRACK
	{ $ast = new AstCastExpr(loc($OPEN_BRACK, $CLOSE_BRACK), $expr.ast, $type.ast); }

	// Enclosed
	| OPEN_BRACK expr CLOSE_BRACK
	{ $ast = $expr.ast; }
	{ $ast.relocate(loc($OPEN_BRACK, $CLOSE_BRACK)); }

	// Postfix (array)
	| e1 = expr OPEN_SBRACK e2 = expr CLOSE_SBRACK
	{ $ast = new AstArrExpr(loc($e1.ast, $CLOSE_SBRACK), $e1.ast, $e2.ast); }

	// Postfix (ptr)
	| e1 = expr CARROT 
	{ $ast = new AstSfxExpr(loc($e1.ast, $CARROT), AstSfxExpr.Oper.PTR, $e1.ast); }

	// Postfix (record)
	| e1 = expr DOT IDENTIFIER
	{ $ast = new AstRecExpr(loc($e1.ast, $IDENTIFIER), $e1.ast, new AstNameExpr(loc($IDENTIFIER), $IDENTIFIER.getText())); }
	
	// Allocations new/delete
	| { AstPfxExpr.Oper oper = null; Location location = null; } 
	(	NEW { oper = AstPfxExpr.Oper.NEW; location = loc($NEW); } |
	 	DEL { oper = AstPfxExpr.Oper.DEL; location = loc($DEL); }
	) expr
	{ $ast = new AstPfxExpr(new Location(location, $expr.ast.location()), oper, $expr.ast);}
	
	// Prefix
	| { AstPfxExpr.Oper oper = null; Location location = null; } 
	(	EXCLAMATION { oper = AstPfxExpr.Oper.NOT; location = loc($EXCLAMATION); } | 
		PLUS { oper = AstPfxExpr.Oper.ADD; location = loc($PLUS); } | 
		MINUS { oper = AstPfxExpr.Oper.SUB; location = loc($MINUS); } | 
		CARROT { oper = AstPfxExpr.Oper.PTR; location = loc($CARROT); }
	) expr
	{ $ast = new AstPfxExpr(new Location(location, $expr.ast.location()), oper, $expr.ast); }
	
	// Infix (mul)
	| e1 = expr {AstBinExpr.Oper oper = null; } 
	(	STAR { oper = AstBinExpr.Oper.MUL; } | 
		SLASH { oper = AstBinExpr.Oper.DIV; } | 
		PERCENT { oper = AstBinExpr.Oper.MOD; }
	) e2 = expr
	{ $ast = new AstBinExpr(loc($e1.ast, $e2.ast), oper, $e1.ast, $e2.ast); }

	// Infix (add)
	| e1 = expr {AstBinExpr.Oper oper = null; }  
	(	PLUS { oper = AstBinExpr.Oper.ADD; } | 
		MINUS { oper = AstBinExpr.Oper.SUB; }
	) e2 = expr
	{ $ast = new AstBinExpr(loc($e1.ast, $e2.ast), oper, $e1.ast, $e2.ast); }
	
	// Infix (boolean)
	| e1 = expr {AstBinExpr.Oper oper = null; } 
	(	IS_EQUAL { oper = AstBinExpr.Oper.EQU; } | 
		NOT_EQUAL { oper = AstBinExpr.Oper.NEQ; } | 
		LESS_THAN { oper = AstBinExpr.Oper.LTH; } | 
		GREATER_THAN { oper = AstBinExpr.Oper.GTH; } |
		LESS_OR_EQUAL { oper = AstBinExpr.Oper.LEQ; } |
		GREATER_OR_EQUAL { oper = AstBinExpr.Oper.GEQ; } 
	) e2 = expr
	{ $ast = new AstBinExpr(loc($e1.ast, $e2.ast), oper, $e1.ast, $e2.ast); }
	
	// Infix (and)
	| e1 = expr (ANDSYM) e2 = expr
	{ $ast = new AstBinExpr(loc($e1.ast, $e2.ast), AstBinExpr.Oper.AND, $e1.ast, $e2.ast); }

	// Infix (or)
	| e1 = expr (PIPE) e2 = expr
	{ $ast = new AstBinExpr(loc($e1.ast, $e2.ast), AstBinExpr.Oper.OR, $e1.ast, $e2.ast); }

	// Where
	| e1 = expr { Vector<AstDecl> decls = new Vector<>(); } 
	 WHERE OPEN_CBRACK (decl { decls.add($decl.ast); })+ CLOSE_CBRACK
	{ $ast = new AstWhereExpr(loc($e1.ast, $CLOSE_CBRACK), $e1.ast, new AstTrees<AstDecl>(decls)); }
	;

stmt returns [AstStmt ast]
	: e1 = expr { $ast = new AstExprStmt($e1.ast.location(), $e1.ast); }
	| e1 = expr EQUALS e2 = expr
	{ $ast = new AstAssignStmt(loc($e1.ast, $e2.ast), $e1.ast, $e2.ast); }

	| IF expr THEN s1 = stmt ELSE s2 = stmt
	{ $ast = new AstIfStmt(loc($IF, $s2.ast), $expr.ast, $s1.ast, $s2.ast); }

	| WHILE expr DO stmt
	{ $ast = new AstWhileStmt(loc($WHILE, $stmt.ast), $expr.ast, $stmt.ast); }
	;